/*  CLOCK version 0.08        Copyright (C) 1992 by Jim Robeson
 *
 *  Function:
 *    This does an on-screen graphical clock, reading the current
 *      date & time out of the BIOS area.
 *    This was written as a self-training exercise in the use of
 *      beginning C graphics. It might help others learn a little, too.
 *    If you jazz this up, it would be nice if you shared your code
 *      with me (send to my BBS, listing below). Thanks.
 *
 *  Other uses:
 *    None.
 *
 *  Execution:
 *    needs EGA/VGA system,
 *    needs EGAVGA.BGI file in current directory
 *
 *  Compiled with:
 *    Borland's Turbo C 2.0.
 *    tcc clock08 graphics.lib
 *
 *  Disclaimer:
 *    This program is contributed to the Public Domain.  It can be
 *    freely used, modified and/or distributed by anyone. The only
 *    thing I ask is that you remember that I am not responsible
 *    for anything that might go wrong through the use of this
 *    program.  I have tested the program enough for my own use.
 *    I also realize that bugs can appear in most every program.
 *    Therefore, YOU USE THIS PROGRAM AT YOUR OWN RISK.
 *
 *  To-do:
 *    a) Figure out how to incorporate EGAVGA.BGI directly into the EXE.
 *    b) Use bit-mapped graphics, so the clock can lay over the top of
 *       any other backgrounds and so the clock hands can be made "pretty".
 *
 *  Enjoy,
 *  Jim Robeson  10/27/92
 *  The Cricket BBS, Pacific Grove CA,  408-373-3773
 */

#include <stdio.h>
#include <string.h>
#include <math.h>            /* for the SIN & COS functions */
#include <dos.h>             /* for the GETDATE & GETTIME functions */
#include <graphics.h>        /* for the screen graphics routines    */
#include <conio.h>           /* for the KBHIT function              */

#define TRUE 1
#define FALSE 0
#define PI  3.14159265358979323846

int init_monitor();      /* prototype */

int now_month;           /* System month (today)              */
int now_day;             /* System day (today)                */
int now_year;            /* System year (today)               */

int now_hour;            /* System hours (time)                 */
int now_min;             /* System minutes (time)               */
int now_sec;             /* System seconds (time)               */
/* int now_hund; */      /* System hundredths of seconds (time) */

int cur_hour;            /* System hours (time)                 */
int cur_min;             /* System minutes (time)               */
double tst_min;          /* System minutes (time)               */
int cur_sec;             /* System seconds (time)               */
double tst_sec;          /* System seconds (time)               */
/* int cur_hund; */      /* System hundredths of seconds (time) */

int graphic_type;

int pntx_sec, pnty_sec;
int pntx_min, pnty_min;
int pntx_hr, pnty_hr;
double angl_sec;
double angl_min;
double angl_hr;

/* ================================================ */
/* === MAIN                                     === */
/* ================================================ */
main()
 {
 struct date today;
 struct time jtime;
/* ------------------------------------------------------------------*/
graphic_type = init_monitor();
if (graphic_type==FALSE) exit (1);  /* abort: non-compatable monitor */

setbkcolor(0);     /* background color         15=white       */
textcolor(1);      /* 0=black, 1=blue, 2=green, 3=cyan, 4=red */
                   /* 5=magneta,6=brown,7=litegrey,8=drkgray  */
                   /* 9=liteblue, 10=litegreen, 11=litecyan   */
                   /* 12=litered, 13=litemagneta, 14=yellow   */
/* setcolor(4);         */
/* circle(319,239,200); */   /* draw a circle */

setcolor(3);                            /* color = cyan */
settextstyle(0,0,4);
outtextxy(319-33,    39-28,     "12");  /* write clock digits */
outtextxy(419-2,     66-30,     "1");
outtextxy(492+3,     139-20,    "2");
outtextxy(519+1,     239-13,    "3");
outtextxy(492,       339-8,     "4");
outtextxy(419,       412,       "5");
outtextxy(319-13,    239+200+1, "6");
outtextxy(219-18,    412+7,     "7");
outtextxy(146-28,    339-4,     "8");
outtextxy(119-25,    239-13,    "9");
outtextxy(146-35,    139-33,    "10");
outtextxy(219-35,    66-35,     "11");

/* ------------------------------------------------------------------*/
 gettime(&jtime);              /* get startup time */
   now_hour=jtime.ti_hour;
   now_min=jtime.ti_min;
   now_sec=jtime.ti_sec;
/* now_hund=jtime.ti_hund; */

 fprintf(stderr,"\n");

loop1:

 getdate(&today);              /* get current date */
   now_year=today.da_year;
   now_month=today.da_mon;
   now_day=today.da_day;

 gettime(&jtime);              /* get current time */
   cur_hour=jtime.ti_hour;
   cur_min=jtime.ti_min;
   cur_sec=jtime.ti_sec;
/* cur_hund=jtime.ti_hund; */
                               /* has time changed ? */
 if (now_hour == cur_hour & now_min == cur_min & now_sec == cur_sec)
   goto loop1;                 /* if no, just wait till it does */
 else
   { now_hour = cur_hour;
     now_min  = cur_min;
     now_sec  = cur_sec;
     tst_min = cur_min;
     tst_sec = cur_sec;
     setcolor(0);           /* black = zap em with BG color */
     line(319,239,pntx_hr,pnty_hr);    /* blank hour hand   */
     line(319,239,pntx_min,pnty_min);  /* blank minute hand */
     line(319,239,pntx_sec,pnty_sec);  /* blank second hand */

     setcolor(14);                        /* yellow */
     if (cur_hour > 12)
        angl_hr = 5*((cur_hour-12)+(tst_min/60)+(tst_sec/3600))*PI/30;
     else
        angl_hr = 5*((cur_hour)+(tst_min/60)+(tst_sec/3600))*PI/30;
     pntx_hr = (sin(angl_hr)*120)+319;    /* do HOUR hand */
     pnty_hr = -(cos(angl_hr)*120)+239;
     line(319,239,pntx_hr,pnty_hr);       /* from middle to where-ever */

     angl_min = (cur_min+(tst_sec/60))*PI/30;   /* do MINUTE hand */
     pntx_min = (sin(angl_min)*160)+319;
     pnty_min = -(cos(angl_min)*160)+239;
     line(319,239,pntx_min,pnty_min);     /* from middle to where-ever */

     setcolor(4);                         /* red  */
     angl_sec = cur_sec*PI/30;            /* do SECOND hand */
     pntx_sec = (sin(angl_sec)*200)+319;
     pnty_sec = -(cos(angl_sec)*200)+239;
     line(319,239,pntx_sec,pnty_sec);     /* from middle to where-ever */

     gotoxy(31,25);
     fprintf(stderr,"%2d-%02d-%d  %2d:%02d:%02d ",now_month,now_day,now_year,now_hour,now_min,now_sec);

     }
 if (kbhit() == 0) goto loop1;   /* if no keys pressed, continue...   */
 restorecrtmode();               /* otherwise, shut down graphics     */
 exit (0);                       /* & get out, errorvalue = 0         */

 }  /* ------------- end of main ------------------------------------ */

/* ============================================= */
/* === INITIALIZE THE MONITOR FOR GRAPHICS   === */
/* ============================================= */

int init_monitor()
{
register int x;
int driver, mode;
int g_error;

x=TRUE;
detectgraph(&driver,&mode);        /* first, see what's there */
if (driver < 0)
  { fprintf(stderr,"No graphics hardware detected !\n");
    x=FALSE;
    }
else
  { printf("Detected graphics driver #%d, mode #%d\n",driver,mode);
    initgraph(&driver,&mode,"");   /* set it up               */
    g_error = graphresult();
    if (g_error < 0)
      { printf("initgraph error: %s.\n",grapherrormsg(g_error));
        x=FALSE;
        }
    }
return(x);
} /* end of init_monitor */

/*------------- End of CLOCK08.C -----------------------------------*/
